#!/usr/bin/env/python
# -*- coding: utf-8 -*-

""" Module for the command line interface to data-fetcher. """
import logging
from argparse import Namespace
from pathlib import Path

# pylint: disable=C0103, E0402, E0611, R0902, R0903, W0703, W1203

from datafetcher.errors import MissingSettingsException, NoProfileException
from datafetcher.project_fetcher import ParallelFetcher

from .locations_report import LocationsReport
from .utilities import get_arg_parser, get_capo_settings, path_is_accessible

_APPLICATION_NAME = "datafetcher"

logger = logging.getLogger(__name__)


class DataFetcher:
    """
        TO EXECUTE ALL DF TESTS: from datafetcher/, just run pytest

        example command line that should work with the correct local profile
        and a current product locator (these are changed whenever the
        database is rebuilt:
        datafetcher --profile local --output-dir ~/Downloads/
        --product-locator \
            uid://evla/execblock/93e1c0cd-76de-4e65-a3f2-d5fe55f386d8 \
            --verbose

        local.properties must have:
        - edu.nrao.archive.workflow.config
            .StartupSettings.temporaryDataDirectory pointing to a locally
            writable temp dir, e.g., /var/tmp
        - edu.nrao.archive.workflow.config.DeliverySettings.hostname must point
            to local computer
        - execution_site must NOT be DSOC or NAASC

    """

    # TODO Some Fine Day: refactor to reduce cognitive complexity
    def __init__(self, args: Namespace, df_capo_settings: dict):
        self.usage = self._build_usage_message()
        if args is None or df_capo_settings is None:
            raise MissingSettingsException()
        self.args = args
        self.settings = df_capo_settings
        try:
            self.verbose = self.args.verbose
        except AttributeError:
            # we don't care; --verbose will be dropped later in WS-179
            pass

        # required arguments
        self.profile = args.profile
        if self.profile is None:
            raise NoProfileException()
        self.output_dir = args.output_dir
        if self.output_dir is None:
            raise MissingSettingsException("output directory option is missing")
        output_dir = Path(self.output_dir)
        if not output_dir.is_dir() or not path_is_accessible(output_dir):
            raise MissingSettingsException(
                f"output location {self.output_dir} inaccessible or not found"
            )

        if args.location_file is not None:
            if args.product_locator is not None:
                raise MissingSettingsException(
                    "required: location file OR product locator -- not both"
                )
            self.location_file = args.location_file
        elif args.product_locator is not None:
            self.product_locator = args.product_locator
        else:
            raise MissingSettingsException(
                "you must specify either a location file or a product locator"
            )

        # optional arguments
        self.is_dry = args.dry_run
        self.force = args.force

        try:
            self.verbose = args.verbose or False
        except AttributeError:
            # verbose is going away soon
            self.verbose = False

        self.locations_report = self._get_locations()
        self.servers_report = self.locations_report.servers_report

    @staticmethod
    def _build_usage_message() -> str:
        usage_str = "Usage:"
        usage_str += """\tdatafetcher [-h]
             (--product-locator PRODUCT_LOCATOR | --location-file LOCATION_FILE)
             [--dry-run] [--output-dir OUTPUT_DIR] [-v, --verbose]
             [--profile PROFILE]\n"""
        return usage_str

    def run(self):
        """
        launch the fetcher
        :return:
        """

        fetcher = ParallelFetcher(self.args, self.settings, self.servers_report)
        fetcher.run()

    def _get_locations(self):
        capo_settings = get_capo_settings(self.profile)
        return LocationsReport(self.args, capo_settings)


def main():
    """This is executed when fetcher is launched
    from the command line.
    """

    logging.basicConfig(level=logging.DEBUG)

    args = get_arg_parser().parse_args()

    settings = get_capo_settings(args.profile)
    DataFetcher(args, settings).run()


if __name__ == "__main__":
    main()
