""" Unit tests for LocationsReportRefactor """

# pylint: disable=W0511, E0401, E0402

import pytest

from datafetcher.locations_report_refactor import (
    LocationsReportRefactor,
    ReportFileParseException,
    UnknownLocatorException,
)

from .fake_archive_service import REPORT_FILES, get_test_data_dir

PROFILE = "docker"


def test_service_returns_expected_report():
    """
    Does FakeArchiveService return a valid locations report?

    :return:
    """
    product_locator = "uid://evla/image/7a546de2-ab1f-4915-a8dc-94f572f9215c"
    args = ["--product-locator", product_locator, "--profile", PROFILE]
    report = LocationsReportRefactor(args)

    assert report.files_report is not None
    assert len(report.servers_report) == 2
    assert len(report.files_report) == 2


def test_empty_file_fails_as_expected():
    """
    When we attempt to create a locations report from a blank
    .json file, do we get the expected error?

    :return:
    """
    file = get_test_data_dir() / REPORT_FILES["empty"]
    args = ["--location-file", str(file), "--profile", PROFILE]
    with pytest.raises(ReportFileParseException):
        LocationsReportRefactor(args)


def test_bad_json_fails_as_expected():
    """
    When we attempt to create a locations report from a badly-formed
    .json file, do we get the expected error?

    :return:
    """
    file = get_test_data_dir() / REPORT_FILES["not_json"]
    args = ["--location-file", str(file), "--profile", PROFILE]
    with pytest.raises(ReportFileParseException):
        LocationsReportRefactor(args)


def test_bad_locator_fails_as_expected():
    """
    An invalid locator should throw UnknownLocatorException

    :return:
    """
    product_locator = "uid://this/is/b00000gus"
    args = ["--product-locator", product_locator, "--profile", PROFILE]
    with pytest.raises(UnknownLocatorException):
        LocationsReportRefactor(args)


def test_missing_file_fails_as_expected():
    """
    If the specified location report file isn't there,
    locations report should throw a FNF

    :return:
    """
    file = get_test_data_dir() / "nonexistent.json"
    args = ["--location-file", str(file), "--profile", PROFILE]
    with pytest.raises(FileNotFoundError):
        LocationsReportRefactor(args)


def test_file_returns_expected_report():
    """
    Can we get a valid locations report from a report file?

    :return:
    """

    file = get_test_data_dir() / REPORT_FILES["vla_eb"]
    args = ["--location-file", str(file), "--profile", PROFILE]
    report = LocationsReportRefactor(args)
    assert len(report.servers_report) == 2
    assert len(report.files_report) == 2


# TODO: other location report tests from test_df_function, test_df_return_codes
