"""
File containing definitions for the notification system of the Workspaces REST API
"""
import smtplib
import chevron

from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config


@view_config(route_name="view_notifications", renderer="json")
def list_notification_templates(request: Request) -> Response:
    """
    Pyramid view to list all available notification templates
    :param request: GET request
    :return:
    """
    response = "Available notification templates:\n"

    for temp in request.notification_info.all_templates():
        response += temp.name + "\n"

    return Response(response)


@view_config(route_name="create_notification", renderer="json")
def create_notification_template(request: Request) -> Response:
    """
    Pyramid view that accepts a request to create a Notification template
    URL: notify/create

    :param request: POST request, expecting JSON parameters ["name", "description", "template"]
    :return:
    """
    name = request.json_body["name"]
    description = request.json_body["description"]
    template = request.json_body["template"]

    # check that we got all the necessary values:
    if len(name) <= 0:
        return Response(
            "Unable to create template.  No value received for template name."
        )

    if len(description) <= 0:
        return Response(
            "Unable to create template.  No value received for template description."
        )

    if len(template) <= 0:
        return Response(
            "Unable to create template.  No value received for template text."
        )

    try:
        new_template = request.notification_info.create_template(
            name, description, template
        )
    except Exception as e:
        return Response(
            "An error occurred saving template: " + name + ".  \nError: " + str(e)
        )

    return Response("Template " + name + " created.")


@view_config(route_name="delete_notification", renderer="json")
def delete_notification_template(request: Request) -> Response:
    """
    Pyramid view to remove a Notification template
    URL: /notify/{template_name}/delete

    :param request: DELETE request
    :return:
    """
    template_name = request.matchdict["template_name"]

    try:
        deleted = request.notification_info.delete_template(template_name)
    except Exception as e:
        return Response(
            "An error occurred deleting template: "
            + template_name
            + ".  \nError: "
            + str(e)
        )

    return Response("Template " + template_name + " deleted.")


@view_config(route_name="send_notification", renderer="json")
def send_notification(request: Request) -> Response:
    """
    Pyramid view to send a notification email
    :param request: POST request, expecting JSON parameters ["destination_email", "subject", "message"]
    :return:
    """

    json_packet = request.json_body
    print(json_packet)

    smtp_server = "smtp.aoc.nrao.edu"
    sender_email = "do-not-reply@nrao.edu"  # Enter your address
    receiver_email = json_packet["destination_email"]  # Enter receiver address
    template_name = request.matchdict["template_name"]

    try:
        template = request.notification_info.lookup_template(template_name)
    except Exception as e:
        return Response(
            "An error occurred looking up template: "
            + template_name
            + "\nerror: "
            + str(e)
        )

    if template is None:
        return Response("No template '" + template_name + "' found.")

    args = {"template": template.content, "data": json_packet}
    try:
        contents = chevron.render(**args)
    except Exception as e:
        return Response(
            "An error occurred rendering template: "
            + str(template)
            + "\nerror: "
            + str(e)
        )

    with smtplib.SMTP(smtp_server) as server:
        r = server.sendmail(sender_email, receiver_email, contents)

    if r == {}:
        return Response("Email sent.\n" + contents)
    else:
        return Response("Unable to send to address(es): " + r)

